@@ -12,7 +12,7 @@ from abc import ABCMeta, abstractmethod
 
 import numpy as np
 from scipy import sparse
-from joblib import Parallel, delayed, effective_n_jobs
+from joblib import Parallel, effective_n_jobs
 
 from ._base import LinearModel, _pre_fit
 from ..base import RegressorMixin, MultiOutputMixin
@@ -25,6 +25,7 @@ from ..utils.fixes import _astype_copy_false, _joblib_parallel_args
 from ..utils.validation import check_is_fitted, _check_sample_weight
 from ..utils.validation import column_or_1d
 from ..utils.validation import _deprecate_positional_args
+from ..utils.fixes import delayed
 
 # mypy error: Module 'sklearn.linear_model' has no attribute '_cd_fast'
 from . import _cd_fast as cd_fast  # type: ignore
@@ -114,7 +115,7 @@ def _alpha_grid(X, y, Xy=None, l1_ratio=1.0, fit_intercept=True,
         If True, the regressors X will be normalized before regression by
         subtracting the mean and dividing by the l2-norm.
         If you wish to standardize, please use
-        :class:`sklearn.preprocessing.StandardScaler` before calling ``fit``
+        :class:`~sklearn.preprocessing.StandardScaler` before calling ``fit``
         on an estimator with ``normalize=False``.
 
     copy_X : bool, default=True
@@ -168,7 +169,8 @@ def _alpha_grid(X, y, Xy=None, l1_ratio=1.0, fit_intercept=True,
                        num=n_alphas)[::-1]
 
 
-def lasso_path(X, y, eps=1e-3, n_alphas=100, alphas=None,
+@_deprecate_positional_args
+def lasso_path(X, y, *, eps=1e-3, n_alphas=100, alphas=None,
                precompute='auto', Xy=None, copy_X=True, coef_init=None,
                verbose=False, return_n_iter=False, positive=False, **params):
     """Compute Lasso path with coordinate descent
@@ -297,8 +299,7 @@ def lasso_path(X, y, eps=1e-3, n_alphas=100, alphas=None,
     [[0.         0.         0.46915237]
      [0.2159048  0.4425765  0.23668876]]
 
-
-    See also
+    See Also
     --------
     lars_path
     Lasso
@@ -313,7 +314,8 @@ def lasso_path(X, y, eps=1e-3, n_alphas=100, alphas=None,
                      positive=positive, return_n_iter=return_n_iter, **params)
 
 
-def enet_path(X, y, l1_ratio=0.5, eps=1e-3, n_alphas=100, alphas=None,
+@_deprecate_positional_args
+def enet_path(X, y, *, l1_ratio=0.5, eps=1e-3, n_alphas=100, alphas=None,
               precompute='auto', Xy=None, copy_X=True, coef_init=None,
               verbose=False, return_n_iter=False, positive=False,
               check_input=True, **params):
@@ -330,7 +332,7 @@ def enet_path(X, y, l1_ratio=0.5, eps=1e-3, n_alphas=100, alphas=None,
 
     For multi-output tasks it is::
 
-        (1 / (2 * n_samples)) * ||Y - XW||^Fro_2
+        (1 / (2 * n_samples)) * ||Y - XW||_Fro^2
         + alpha * l1_ratio * ||W||_21
         + 0.5 * alpha * (1 - l1_ratio) * ||W||_Fro^2
 
@@ -396,8 +398,9 @@ def enet_path(X, y, l1_ratio=0.5, eps=1e-3, n_alphas=100, alphas=None,
         (Only allowed when ``y.ndim == 1``).
 
     check_input : bool, default=True
-        Skip input validation checks, including the Gram matrix when provided
-        assuming there are handled by the caller when check_input=False.
+        If set to False, the input validation checks are skipped (including the
+        Gram matrix when provided). It is assumed that they are handled
+        by the caller.
 
     **params : kwargs
         Keyword arguments passed to the coordinate descent solver.
@@ -564,7 +567,7 @@ class ElasticNet(MultiOutputMixin, RegressorMixin, LinearModel):
     If you are interested in controlling the L1 and L2 penalty
     separately, keep in mind that this is equivalent to::
 
-            a * L1 + b * L2
+            a * ||w||_1 + 0.5 * b * ||w||_2^2
 
     where::
 
@@ -602,17 +605,17 @@ class ElasticNet(MultiOutputMixin, RegressorMixin, LinearModel):
         If True, the regressors X will be normalized before regression by
         subtracting the mean and dividing by the l2-norm.
         If you wish to standardize, please use
-        :class:`sklearn.preprocessing.StandardScaler` before calling ``fit``
+        :class:`~sklearn.preprocessing.StandardScaler` before calling ``fit``
         on an estimator with ``normalize=False``.
 
     precompute : bool or array-like of shape (n_features, n_features),\
                  default=False
         Whether to use a precomputed Gram matrix to speed up
         calculations. The Gram matrix can also be passed as argument.
-        For sparse input this option is always ``True`` to preserve sparsity.
+        For sparse input this option is always ``False`` to preserve sparsity.
 
     max_iter : int, default=1000
-        The maximum number of iterations
+        The maximum number of iterations.
 
     copy_X : bool, default=True
         If ``True``, X will be copied; else, it may be overwritten.
@@ -646,19 +649,23 @@ class ElasticNet(MultiOutputMixin, RegressorMixin, LinearModel):
     Attributes
     ----------
     coef_ : ndarray of shape (n_features,) or (n_targets, n_features)
-        parameter vector (w in the cost function formula)
+        Parameter vector (w in the cost function formula).
 
-    sparse_coef_ : sparse matrix of shape (n_features, 1) or \
-            (n_targets, n_features)
-        ``sparse_coef_`` is a readonly property derived from ``coef_``
+    sparse_coef_ : sparse matrix of shape (n_features,) or \
+            (n_tasks, n_features)
+        Sparse representation of the `coef_`.
 
     intercept_ : float or ndarray of shape (n_targets,)
-        independent term in decision function.
+        Independent term in decision function.
 
     n_iter_ : list of int
-        number of iterations run by the coordinate descent solver to reach
+        Number of iterations run by the coordinate descent solver to reach
         the specified tolerance.
 
+    dual_gap_ : float or ndarray of shape (n_targets,)
+        Given param alpha, the dual gaps at the end of the optimization,
+        same shape as each observation of y.
+
     Examples
     --------
     >>> from sklearn.linear_model import ElasticNet
@@ -681,12 +688,12 @@ class ElasticNet(MultiOutputMixin, RegressorMixin, LinearModel):
     To avoid unnecessary memory duplication the X argument of the fit method
     should be directly passed as a Fortran-contiguous numpy array.
 
-    See also
+    See Also
     --------
     ElasticNetCV : Elastic net model with best model selection by
         cross-validation.
-    SGDRegressor: implements elastic net regression with incremental training.
-    SGDClassifier: implements logistic regression with elastic net penalty
+    SGDRegressor : Implements elastic net regression with incremental training.
+    SGDClassifier : Implements logistic regression with elastic net penalty
         (``SGDClassifier(loss="log", penalty="elasticnet")``).
     """
     path = staticmethod(enet_path)
@@ -715,15 +722,17 @@ class ElasticNet(MultiOutputMixin, RegressorMixin, LinearModel):
         Parameters
         ----------
         X : {ndarray, sparse matrix} of (n_samples, n_features)
-            Data
+            Data.
 
         y : {ndarray, sparse matrix} of shape (n_samples,) or \
             (n_samples, n_targets)
-            Target. Will be cast to X's dtype if necessary
+            Target. Will be cast to X's dtype if necessary.
 
         sample_weight : float or array-like of shape (n_samples,), default=None
             Sample weight.
 
+            .. versionadded:: 0.23
+
         check_input : bool, default=True
             Allow to bypass several input checking.
             Don't use this parameter unless you know what you do.
@@ -748,6 +757,11 @@ class ElasticNet(MultiOutputMixin, RegressorMixin, LinearModel):
             raise ValueError('precompute should be one of True, False or'
                              ' array-like. Got %r' % self.precompute)
 
+        if (not isinstance(self.l1_ratio, numbers.Number) or
+                self.l1_ratio < 0 or self.l1_ratio > 1):
+            raise ValueError("l1_ratio must be between 0 and 1; "
+                             f"got l1_ratio={self.l1_ratio}")
+
         # Remember if X is copied
         X_copied = False
         # We expect X and y to be float64 or float32 Fortran ordered arrays
@@ -861,11 +875,11 @@ class ElasticNet(MultiOutputMixin, RegressorMixin, LinearModel):
 
     @property
     def sparse_coef_(self):
-        """ sparse representation of the fitted ``coef_`` """
+        """Sparse representation of the fitted `coef_`."""
         return sparse.csr_matrix(self.coef_)
 
     def _decision_function(self, X):
-        """Decision function of the linear model
+        """Decision function of the linear model.
 
         Parameters
         ----------
@@ -874,7 +888,7 @@ class ElasticNet(MultiOutputMixin, RegressorMixin, LinearModel):
         Returns
         -------
         T : ndarray of shape (n_samples,)
-            The predicted decision function
+            The predicted decision function.
         """
         check_is_fitted(self)
         if sparse.isspmatrix(X):
@@ -918,21 +932,20 @@ class Lasso(ElasticNet):
         If True, the regressors X will be normalized before regression by
         subtracting the mean and dividing by the l2-norm.
         If you wish to standardize, please use
-        :class:`sklearn.preprocessing.StandardScaler` before calling ``fit``
+        :class:`~sklearn.preprocessing.StandardScaler` before calling ``fit``
         on an estimator with ``normalize=False``.
 
-    precompute : 'auto', bool or array-like of shape (n_features, n_features),\
+    precompute : bool or array-like of shape (n_features, n_features),\
                  default=False
         Whether to use a precomputed Gram matrix to speed up
-        calculations. If set to ``'auto'`` let us decide. The Gram
-        matrix can also be passed as argument. For sparse input
-        this option is always ``True`` to preserve sparsity.
+        calculations. The Gram matrix can also be passed as argument.
+        For sparse input this option is always ``False`` to preserve sparsity.
 
     copy_X : bool, default=True
         If ``True``, X will be copied; else, it may be overwritten.
 
     max_iter : int, default=1000
-        The maximum number of iterations
+        The maximum number of iterations.
 
     tol : float, default=1e-4
         The tolerance for the optimization: if the updates are
@@ -963,17 +976,21 @@ class Lasso(ElasticNet):
     Attributes
     ----------
     coef_ : ndarray of shape (n_features,) or (n_targets, n_features)
-        parameter vector (w in the cost function formula)
+        Parameter vector (w in the cost function formula).
+
+    dual_gap_ : float or ndarray of shape (n_targets,)
+        Given param alpha, the dual gaps at the end of the optimization,
+        same shape as each observation of y.
 
     sparse_coef_ : sparse matrix of shape (n_features, 1) or \
             (n_targets, n_features)
-        ``sparse_coef_`` is a readonly property derived from ``coef_``
+        Readonly property derived from ``coef_``.
 
     intercept_ : float or ndarray of shape (n_targets,)
-        independent term in decision function.
+        Independent term in decision function.
 
     n_iter_ : int or list of int
-        number of iterations run by the coordinate descent solver to reach
+        Number of iterations run by the coordinate descent solver to reach
         the specified tolerance.
 
     Examples
@@ -987,7 +1004,7 @@ class Lasso(ElasticNet):
     >>> print(clf.intercept_)
     0.15...
 
-    See also
+    See Also
     --------
     lars_path
     lasso_path
@@ -1023,7 +1040,7 @@ class Lasso(ElasticNet):
 
 def _path_residuals(X, y, train, test, path, path_params, alphas=None,
                     l1_ratio=1, X_order=None, dtype=None):
-    """Returns the MSE for the models computed by 'path'
+    """Returns the MSE for the models computed by 'path'.
 
     Parameters
     ----------
@@ -1031,20 +1048,20 @@ def _path_residuals(X, y, train, test, path, path_params, alphas=None,
         Training data.
 
     y : array-like of shape (n_samples,) or (n_samples, n_targets)
-        Target values
+        Target values.
 
     train : list of indices
-        The indices of the train set
+        The indices of the train set.
 
     test : list of indices
-        The indices of the test set
+        The indices of the test set.
 
     path : callable
-        function returning a list of models on the path. See
-        enet_path for an example of signature
+        Function returning a list of models on the path. See
+        enet_path for an example of signature.
 
     path_params : dictionary
-        Parameters passed to the path function
+        Parameters passed to the path function.
 
     alphas : array-like, default=None
         Array of float that is used for cross-validation. If not
@@ -1058,11 +1075,11 @@ def _path_residuals(X, y, train, test, path, path_params, alphas=None,
 
     X_order : {'F', 'C'}, default=None
         The order of the arrays expected by the path function to
-        avoid memory copies
+        avoid memory copies.
 
     dtype : a numpy dtype, default=None
         The dtype of the arrays expected by the path function to
-        avoid memory copies
+        avoid memory copies.
     """
     X_train = X[train]
     y_train = y[train]
@@ -1129,7 +1146,7 @@ def _path_residuals(X, y, train, test, path, path_params, alphas=None,
 
 
 class LinearModelCV(MultiOutputMixin, LinearModel, metaclass=ABCMeta):
-    """Base class for iterative model fitting along a regularization path"""
+    """Base class for iterative model fitting along a regularization path."""
 
     @abstractmethod
     def __init__(self, eps=1e-3, n_alphas=100, alphas=None, fit_intercept=True,
@@ -1152,8 +1169,16 @@ class LinearModelCV(MultiOutputMixin, LinearModel, metaclass=ABCMeta):
         self.random_state = random_state
         self.selection = selection
 
+    @abstractmethod
+    def _get_estimator(self):
+        """Model to be fitted after the best alpha has been determined."""
+
+    @abstractmethod
+    def _is_multitask(self):
+        """Bool indicating if class is meant for multidimensional target."""
+
     def fit(self, X, y):
-        """Fit linear model with coordinate descent
+        """Fit linear model with coordinate descent.
 
         Fit is on grid of alphas and best alpha estimated by cross-validation.
 
@@ -1165,7 +1190,7 @@ class LinearModelCV(MultiOutputMixin, LinearModel, metaclass=ABCMeta):
             X can be sparse.
 
         y : array-like of shape (n_samples,) or (n_samples, n_targets)
-            Target values
+            Target values.
         """
         # This makes sure that there is no duplication in memory.
         # Dealing right with copy_X is important in the following:
@@ -1216,19 +1241,10 @@ class LinearModelCV(MultiOutputMixin, LinearModel, metaclass=ABCMeta):
         if y.shape[0] == 0:
             raise ValueError("y has 0 samples: %r" % y)
 
-        if hasattr(self, 'l1_ratio'):
-            model_str = 'ElasticNet'
-        else:
-            model_str = 'Lasso'
-
-        if isinstance(self, ElasticNetCV) or isinstance(self, LassoCV):
-            if model_str == 'ElasticNet':
-                model = ElasticNet()
-            else:
-                model = Lasso()
+        if not self._is_multitask():
             if y.ndim > 1 and y.shape[1] > 1:
                 raise ValueError("For multi-task outputs, use "
-                                 "MultiTask%sCV" % (model_str))
+                                 "MultiTask%s" % self.__class__.__name__)
             y = column_or_1d(y, warn=True)
         else:
             if sparse.isspmatrix(X):
@@ -1236,11 +1252,9 @@ class LinearModelCV(MultiOutputMixin, LinearModel, metaclass=ABCMeta):
                                 "passed")
             elif y.ndim == 1:
                 raise ValueError("For mono-task outputs, use "
-                                 "%sCV" % (model_str))
-            if model_str == 'ElasticNet':
-                model = MultiTaskElasticNet()
-            else:
-                model = MultiTaskLasso()
+                                 "%sCV" % self.__class__.__name__[9:])
+
+        model = self._get_estimator()
 
         if self.selection not in ["random", "cyclic"]:
             raise ValueError("selection should be either random or cyclic.")
@@ -1361,14 +1375,14 @@ class LassoCV(RegressorMixin, LinearModelCV):
         ``alpha_min / alpha_max = 1e-3``.
 
     n_alphas : int, default=100
-        Number of alphas along the regularization path
+        Number of alphas along the regularization path.
 
     alphas : ndarray, default=None
         List of alphas where to compute the models.
-        If ``None`` alphas are set automatically
+        If ``None`` alphas are set automatically.
 
     fit_intercept : bool, default=True
-        whether to calculate the intercept for this model. If set
+        Whether to calculate the intercept for this model. If set
         to false, no intercept will be used in calculations
         (i.e. data is expected to be centered).
 
@@ -1377,7 +1391,7 @@ class LassoCV(RegressorMixin, LinearModelCV):
         If True, the regressors X will be normalized before regression by
         subtracting the mean and dividing by the l2-norm.
         If you wish to standardize, please use
-        :class:`sklearn.preprocessing.StandardScaler` before calling ``fit``
+        :class:`~sklearn.preprocessing.StandardScaler` before calling ``fit``
         on an estimator with ``normalize=False``.
 
     precompute : 'auto', bool or array-like of shape (n_features, n_features),\
@@ -1387,7 +1401,7 @@ class LassoCV(RegressorMixin, LinearModelCV):
         matrix can also be passed as argument.
 
     max_iter : int, default=1000
-        The maximum number of iterations
+        The maximum number of iterations.
 
     tol : float, default=1e-4
         The tolerance for the optimization: if the updates are
@@ -1425,7 +1439,7 @@ class LassoCV(RegressorMixin, LinearModelCV):
         for more details.
 
     positive : bool, default=False
-        If positive, restrict regression coefficients to be positive
+        If positive, restrict regression coefficients to be positive.
 
     random_state : int, RandomState instance, default=None
         The seed of the pseudo random number generator that selects a random
@@ -1442,26 +1456,26 @@ class LassoCV(RegressorMixin, LinearModelCV):
     Attributes
     ----------
     alpha_ : float
-        The amount of penalization chosen by cross validation
+        The amount of penalization chosen by cross validation.
 
     coef_ : ndarray of shape (n_features,) or (n_targets, n_features)
-        parameter vector (w in the cost function formula)
+        Parameter vector (w in the cost function formula).
 
     intercept_ : float or ndarray of shape (n_targets,)
-        independent term in decision function.
+        Independent term in decision function.
 
     mse_path_ : ndarray of shape (n_alphas, n_folds)
-        mean square error for the test set on each fold, varying alpha
+        Mean square error for the test set on each fold, varying alpha.
 
     alphas_ : ndarray of shape (n_alphas,)
-        The grid of alphas used for fitting
+        The grid of alphas used for fitting.
 
     dual_gap_ : float or ndarray of shape (n_targets,)
         The dual gap at the end of the optimization for the optimal alpha
         (``alpha_``).
 
     n_iter_ : int
-        number of iterations run by the coordinate descent solver to reach
+        Number of iterations run by the coordinate descent solver to reach
         the specified tolerance for the optimal alpha.
 
     Examples
@@ -1484,7 +1498,7 @@ class LassoCV(RegressorMixin, LinearModelCV):
     To avoid unnecessary memory duplication the X argument of the fit method
     should be directly passed as a Fortran-contiguous numpy array.
 
-    See also
+    See Also
     --------
     lars_path
     lasso_path
@@ -1507,6 +1521,12 @@ class LassoCV(RegressorMixin, LinearModelCV):
             cv=cv, verbose=verbose, n_jobs=n_jobs, positive=positive,
             random_state=random_state, selection=selection)
 
+    def _get_estimator(self):
+        return Lasso()
+
+    def _is_multitask(self):
+        return False
+
     def _more_tags(self):
         return {'multioutput': False}
 
@@ -1530,7 +1550,7 @@ class ElasticNetCV(RegressorMixin, LinearModelCV):
         prediction score is used. Note that a good choice of list of
         values for l1_ratio is often to put more values close to 1
         (i.e. Lasso) and less close to 0 (i.e. Ridge), as in ``[.1, .5, .7,
-        .9, .95, .99, 1]``
+        .9, .95, .99, 1]``.
 
     eps : float, default=1e-3
         Length of the path. ``eps=1e-3`` means that
@@ -1541,10 +1561,10 @@ class ElasticNetCV(RegressorMixin, LinearModelCV):
 
     alphas : ndarray, default=None
         List of alphas where to compute the models.
-        If None alphas are set automatically
+        If None alphas are set automatically.
 
     fit_intercept : bool, default=True
-        whether to calculate the intercept for this model. If set
+        Whether to calculate the intercept for this model. If set
         to false, no intercept will be used in calculations
         (i.e. data is expected to be centered).
 
@@ -1553,7 +1573,7 @@ class ElasticNetCV(RegressorMixin, LinearModelCV):
         If True, the regressors X will be normalized before regression by
         subtracting the mean and dividing by the l2-norm.
         If you wish to standardize, please use
-        :class:`sklearn.preprocessing.StandardScaler` before calling ``fit``
+        :class:`~sklearn.preprocessing.StandardScaler` before calling ``fit``
         on an estimator with ``normalize=False``.
 
     precompute : 'auto', bool or array-like of shape (n_features, n_features),\
@@ -1563,7 +1583,7 @@ class ElasticNetCV(RegressorMixin, LinearModelCV):
         matrix can also be passed as argument.
 
     max_iter : int, default=1000
-        The maximum number of iterations
+        The maximum number of iterations.
 
     tol : float, default=1e-4
         The tolerance for the optimization: if the updates are
@@ -1618,14 +1638,14 @@ class ElasticNetCV(RegressorMixin, LinearModelCV):
     Attributes
     ----------
     alpha_ : float
-        The amount of penalization chosen by cross validation
+        The amount of penalization chosen by cross validation.
 
     l1_ratio_ : float
         The compromise between l1 and l2 penalization chosen by
-        cross validation
+        cross validation.
 
     coef_ : ndarray of shape (n_features,) or (n_targets, n_features)
-        Parameter vector (w in the cost function formula),
+        Parameter vector (w in the cost function formula).
 
     intercept_ : float or ndarray of shape (n_targets, n_features)
         Independent term in the decision function.
@@ -1637,8 +1657,11 @@ class ElasticNetCV(RegressorMixin, LinearModelCV):
     alphas_ : ndarray of shape (n_alphas,) or (n_l1_ratio, n_alphas)
         The grid of alphas used for fitting, for each l1_ratio.
 
+    dual_gap_ : float
+        The dual gaps at the end of the optimization for the optimal alpha.
+
     n_iter_ : int
-        number of iterations run by the coordinate descent solver to reach
+        Number of iterations run by the coordinate descent solver to reach
         the specified tolerance for the optimal alpha.
 
     Examples
@@ -1684,7 +1707,7 @@ class ElasticNetCV(RegressorMixin, LinearModelCV):
 
         alpha = a + b and l1_ratio = a / (a + b).
 
-    See also
+    See Also
     --------
     enet_path
     ElasticNet
@@ -1715,6 +1738,12 @@ class ElasticNetCV(RegressorMixin, LinearModelCV):
         self.random_state = random_state
         self.selection = selection
 
+    def _get_estimator(self):
+        return ElasticNet()
+
+    def _is_multitask(self):
+        return False
+
     def _more_tags(self):
         return {'multioutput': False}
 
@@ -1723,7 +1752,8 @@ class ElasticNetCV(RegressorMixin, LinearModelCV):
 
 
 class MultiTaskElasticNet(Lasso):
-    """Multi-task ElasticNet model trained with L1/L2 mixed-norm as regularizer
+    """Multi-task ElasticNet model trained with L1/L2 mixed-norm as
+    regularizer.
 
     The optimization objective for MultiTaskElasticNet is::
 
@@ -1742,7 +1772,7 @@ class MultiTaskElasticNet(Lasso):
     Parameters
     ----------
     alpha : float, default=1.0
-        Constant that multiplies the L1/L2 term. Defaults to 1.0
+        Constant that multiplies the L1/L2 term. Defaults to 1.0.
 
     l1_ratio : float, default=0.5
         The ElasticNet mixing parameter, with 0 < l1_ratio <= 1.
@@ -1751,7 +1781,7 @@ class MultiTaskElasticNet(Lasso):
         For ``0 < l1_ratio < 1``, the penalty is a combination of L1/L2 and L2.
 
     fit_intercept : bool, default=True
-        whether to calculate the intercept for this model. If set
+        Whether to calculate the intercept for this model. If set
         to false, no intercept will be used in calculations
         (i.e. data is expected to be centered).
 
@@ -1760,14 +1790,14 @@ class MultiTaskElasticNet(Lasso):
         If True, the regressors X will be normalized before regression by
         subtracting the mean and dividing by the l2-norm.
         If you wish to standardize, please use
-        :class:`sklearn.preprocessing.StandardScaler` before calling ``fit``
+        :class:`~sklearn.preprocessing.StandardScaler` before calling ``fit``
         on an estimator with ``normalize=False``.
 
     copy_X : bool, default=True
         If ``True``, X will be copied; else, it may be overwritten.
 
     max_iter : int, default=1000
-        The maximum number of iterations
+        The maximum number of iterations.
 
     tol : float, default=1e-4
         The tolerance for the optimization: if the updates are
@@ -1803,9 +1833,19 @@ class MultiTaskElasticNet(Lasso):
         Note that ``coef_`` stores the transpose of ``W``, ``W.T``.
 
     n_iter_ : int
-        number of iterations run by the coordinate descent solver to reach
+        Number of iterations run by the coordinate descent solver to reach
         the specified tolerance.
 
+    dual_gap_ : float
+        The dual gaps at the end of the optimization.
+
+    eps_ : float
+        The tolerance scaled scaled by the variance of the target `y`.
+
+    sparse_coef_ : sparse matrix of shape (n_features,) or \
+            (n_tasks, n_features)
+        Sparse representation of the `coef_`.
+
     Examples
     --------
     >>> from sklearn import linear_model
@@ -1818,9 +1858,9 @@ class MultiTaskElasticNet(Lasso):
     >>> print(clf.intercept_)
     [0.0872422 0.0872422]
 
-    See also
+    See Also
     --------
-    MultiTaskElasticNet : Multi-task L1/L2 ElasticNet with built-in
+    MultiTaskElasticNetCV : Multi-task L1/L2 ElasticNet with built-in
         cross-validation.
     ElasticNet
     MultiTaskLasso
@@ -1853,9 +1893,9 @@ class MultiTaskElasticNet(Lasso):
         Parameters
         ----------
         X : ndarray of shape (n_samples, n_features)
-            Data
+            Data.
         y : ndarray of shape (n_samples, n_tasks)
-            Target. Will be cast to X's dtype if necessary
+            Target. Will be cast to X's dtype if necessary.
 
         Notes
         -----
@@ -1938,10 +1978,10 @@ class MultiTaskLasso(MultiTaskElasticNet):
     Parameters
     ----------
     alpha : float, default=1.0
-        Constant that multiplies the L1/L2 term. Defaults to 1.0
+        Constant that multiplies the L1/L2 term. Defaults to 1.0.
 
     fit_intercept : bool, default=True
-        whether to calculate the intercept for this model. If set
+        Whether to calculate the intercept for this model. If set
         to false, no intercept will be used in calculations
         (i.e. data is expected to be centered).
 
@@ -1950,14 +1990,14 @@ class MultiTaskLasso(MultiTaskElasticNet):
         If True, the regressors X will be normalized before regression by
         subtracting the mean and dividing by the l2-norm.
         If you wish to standardize, please use
-        :class:`sklearn.preprocessing.StandardScaler` before calling ``fit``
+        :class:`~sklearn.preprocessing.StandardScaler` before calling ``fit``
         on an estimator with ``normalize=False``.
 
     copy_X : bool, default=True
         If ``True``, X will be copied; else, it may be overwritten.
 
     max_iter : int, default=1000
-        The maximum number of iterations
+        The maximum number of iterations.
 
     tol : float, default=1e-4
         The tolerance for the optimization: if the updates are
@@ -1989,12 +2029,22 @@ class MultiTaskLasso(MultiTaskElasticNet):
         Note that ``coef_`` stores the transpose of ``W``, ``W.T``.
 
     intercept_ : ndarray of shape (n_tasks,)
-        independent term in decision function.
+        Independent term in decision function.
 
     n_iter_ : int
-        number of iterations run by the coordinate descent solver to reach
+        Number of iterations run by the coordinate descent solver to reach
         the specified tolerance.
 
+    dual_gap_ : ndarray of shape (n_alphas,)
+        The dual gaps at the end of the optimization for each alpha.
+
+    eps_ : float
+        The tolerance scaled scaled by the variance of the target `y`.
+
+    sparse_coef_ : sparse matrix of shape (n_features,) or \
+            (n_tasks, n_features)
+        Sparse representation of the `coef_`.
+
     Examples
     --------
     >>> from sklearn import linear_model
@@ -2007,9 +2057,9 @@ class MultiTaskLasso(MultiTaskElasticNet):
     >>> print(clf.intercept_)
     [-0.41888636 -0.87382323]
 
-    See also
+    See Also
     --------
-    MultiTaskLasso : Multi-task L1/L2 Lasso with built-in cross-validation
+    MultiTaskLasso : Multi-task L1/L2 Lasso with built-in cross-validation.
     Lasso
     MultiTaskElasticNet
 
@@ -2076,14 +2126,14 @@ class MultiTaskElasticNetCV(RegressorMixin, LinearModelCV):
         ``alpha_min / alpha_max = 1e-3``.
 
     n_alphas : int, default=100
-        Number of alphas along the regularization path
+        Number of alphas along the regularization path.
 
     alphas : array-like, default=None
         List of alphas where to compute the models.
         If not provided, set automatically.
 
     fit_intercept : bool, default=True
-        whether to calculate the intercept for this model. If set
+        Whether to calculate the intercept for this model. If set
         to false, no intercept will be used in calculations
         (i.e. data is expected to be centered).
 
@@ -2092,11 +2142,11 @@ class MultiTaskElasticNetCV(RegressorMixin, LinearModelCV):
         If True, the regressors X will be normalized before regression by
         subtracting the mean and dividing by the l2-norm.
         If you wish to standardize, please use
-        :class:`sklearn.preprocessing.StandardScaler` before calling ``fit``
+        :class:`~sklearn.preprocessing.StandardScaler` before calling ``fit``
         on an estimator with ``normalize=False``.
 
     max_iter : int, default=1000
-        The maximum number of iterations
+        The maximum number of iterations.
 
     tol : float, default=1e-4
         The tolerance for the optimization: if the updates are
@@ -2156,22 +2206,25 @@ class MultiTaskElasticNetCV(RegressorMixin, LinearModelCV):
         Note that ``coef_`` stores the transpose of ``W``, ``W.T``.
 
     alpha_ : float
-        The amount of penalization chosen by cross validation
+        The amount of penalization chosen by cross validation.
 
     mse_path_ : ndarray of shape (n_alphas, n_folds) or \
                 (n_l1_ratio, n_alphas, n_folds)
-        mean square error for the test set on each fold, varying alpha
+        Mean square error for the test set on each fold, varying alpha.
 
     alphas_ : ndarray of shape (n_alphas,) or (n_l1_ratio, n_alphas)
-        The grid of alphas used for fitting, for each l1_ratio
+        The grid of alphas used for fitting, for each l1_ratio.
 
     l1_ratio_ : float
-        best l1_ratio obtained by cross-validation.
+        Best l1_ratio obtained by cross-validation.
 
     n_iter_ : int
-        number of iterations run by the coordinate descent solver to reach
+        Number of iterations run by the coordinate descent solver to reach
         the specified tolerance for the optimal alpha.
 
+    dual_gap_ : float
+        The dual gap at the end of the optimization for the optimal alpha.
+
     Examples
     --------
     >>> from sklearn import linear_model
@@ -2185,7 +2238,7 @@ class MultiTaskElasticNetCV(RegressorMixin, LinearModelCV):
     >>> print(clf.intercept_)
     [0.00166409 0.00166409]
 
-    See also
+    See Also
     --------
     MultiTaskElasticNet
     ElasticNetCV
@@ -2221,6 +2274,12 @@ class MultiTaskElasticNetCV(RegressorMixin, LinearModelCV):
         self.random_state = random_state
         self.selection = selection
 
+    def _get_estimator(self):
+        return MultiTaskElasticNet()
+
+    def _is_multitask(self):
+        return True
+
     def _more_tags(self):
         return {'multioutput_only': True}
 
@@ -2251,14 +2310,14 @@ class MultiTaskLassoCV(RegressorMixin, LinearModelCV):
         ``alpha_min / alpha_max = 1e-3``.
 
     n_alphas : int, default=100
-        Number of alphas along the regularization path
+        Number of alphas along the regularization path.
 
     alphas : array-like, default=None
         List of alphas where to compute the models.
         If not provided, set automatically.
 
     fit_intercept : bool, default=True
-        whether to calculate the intercept for this model. If set
+        Whether to calculate the intercept for this model. If set
         to false, no intercept will be used in calculations
         (i.e. data is expected to be centered).
 
@@ -2267,7 +2326,7 @@ class MultiTaskLassoCV(RegressorMixin, LinearModelCV):
         If True, the regressors X will be normalized before regression by
         subtracting the mean and dividing by the l2-norm.
         If you wish to standardize, please use
-        :class:`sklearn.preprocessing.StandardScaler` before calling ``fit``
+        :class:`~sklearn.preprocessing.StandardScaler` before calling ``fit``
         on an estimator with ``normalize=False``.
 
     max_iter : int, default=1000
@@ -2331,18 +2390,21 @@ class MultiTaskLassoCV(RegressorMixin, LinearModelCV):
         Note that ``coef_`` stores the transpose of ``W``, ``W.T``.
 
     alpha_ : float
-        The amount of penalization chosen by cross validation
+        The amount of penalization chosen by cross validation.
 
     mse_path_ : ndarray of shape (n_alphas, n_folds)
-        mean square error for the test set on each fold, varying alpha
+        Mean square error for the test set on each fold, varying alpha.
 
     alphas_ : ndarray of shape (n_alphas,)
         The grid of alphas used for fitting.
 
     n_iter_ : int
-        number of iterations run by the coordinate descent solver to reach
+        Number of iterations run by the coordinate descent solver to reach
         the specified tolerance for the optimal alpha.
 
+    dual_gap_ : float
+        The dual gap at the end of the optimization for the optimal alpha.
+
     Examples
     --------
     >>> from sklearn.linear_model import MultiTaskLassoCV
@@ -2357,7 +2419,7 @@ class MultiTaskLassoCV(RegressorMixin, LinearModelCV):
     >>> reg.predict(X[:1,])
     array([[153.7971...,  94.9015...]])
 
-    See also
+    See Also
     --------
     MultiTaskElasticNet
     ElasticNetCV
@@ -2385,5 +2447,11 @@ class MultiTaskLassoCV(RegressorMixin, LinearModelCV):
             cv=cv, verbose=verbose, n_jobs=n_jobs, random_state=random_state,
             selection=selection)
 
+    def _get_estimator(self):
+        return MultiTaskLasso()
+
+    def _is_multitask(self):
+        return True
+
     def _more_tags(self):
         return {'multioutput_only': True}
