@@ -1,7 +1,5 @@
 import warnings
 
-from distutils.version import LooseVersion
-
 import numpy as np
 import pytest
 from scipy import linalg
@@ -14,6 +12,7 @@ from sklearn.utils._testing import assert_raises
 from sklearn.utils._testing import ignore_warnings
 from sklearn.utils._testing import assert_warns
 from sklearn.utils._testing import TempMemmap
+from sklearn.utils.fixes import np_version, parse_version
 from sklearn.exceptions import ConvergenceWarning
 from sklearn import linear_model, datasets
 from sklearn.linear_model._least_angle import _lars_path_residues
@@ -39,7 +38,8 @@ def test_simple():
         sys.stdout = StringIO()
 
         _, _, coef_path_ = linear_model.lars_path(
-            X, y, method='lar', verbose=10)
+            X, y, method="lar", verbose=10
+        )
 
         sys.stdout = old_stdout
 
@@ -61,8 +61,7 @@ def test_simple():
 def test_simple_precomputed():
     # The same, with precomputed Gram matrix
 
-    _, _, coef_path_ = linear_model.lars_path(
-        X, y, Gram=G, method='lar')
+    _, _, coef_path_ = linear_model.lars_path(X, y, Gram=G, method="lar")
 
     for i, coef_ in enumerate(coef_path_.T):
         res = y - np.dot(X, coef_)
@@ -83,8 +82,8 @@ def _assert_same_lars_path_result(output1, output2):
         assert_allclose(o1, o2)
 
 
-@pytest.mark.parametrize('method', ['lar', 'lasso'])
-@pytest.mark.parametrize('return_path', [True, False])
+@pytest.mark.parametrize("method", ["lar", "lasso"])
+@pytest.mark.parametrize("return_path", [True, False])
 def test_lars_path_gram_equivalent(method, return_path):
     _assert_same_lars_path_result(
         linear_model.lars_path_gram(
@@ -106,7 +105,7 @@ def test_all_precomputed():
     # Test that lars_path with precomputed Gram and Xy gives the right answer
     G = np.dot(X.T, X)
     Xy = np.dot(X.T, y)
-    for method in 'lar', 'lasso':
+    for method in "lar", "lasso":
         output = linear_model.lars_path(X, y, method=method)
         output_pre = linear_model.lars_path(X, y, Gram=G, Xy=Xy,
                                             method=method)
@@ -123,7 +122,7 @@ def test_lars_lstsq():
     clf = linear_model.LassoLars(alpha=0.)
     clf.fit(X1, y)
     # Avoid FutureWarning about default value change when numpy >= 1.14
-    rcond = None if LooseVersion(np.__version__) >= '1.14' else -1
+    rcond = None if np_version >= parse_version('1.14') else -1
     coef_lstsq = np.linalg.lstsq(X1, y, rcond=rcond)[0]
     assert_array_almost_equal(clf.coef_, coef_lstsq)
 
@@ -164,10 +163,10 @@ def test_collinearity():
 
 def test_no_path():
     # Test that the ``return_path=False`` option returns the correct output
-    alphas_, _, coef_path_ = linear_model.lars_path(
-        X, y, method='lar')
+    alphas_, _, coef_path_ = linear_model.lars_path(X, y, method="lar")
     alpha_, _, coef = linear_model.lars_path(
-        X, y, method='lar', return_path=False)
+        X, y, method="lar", return_path=False
+    )
 
     assert_array_almost_equal(coef, coef_path_[:, -1])
     assert alpha_ == alphas_[-1]
@@ -175,10 +174,10 @@ def test_no_path():
 
 def test_no_path_precomputed():
     # Test that the ``return_path=False`` option with Gram remains correct
-    alphas_, _, coef_path_ = linear_model.lars_path(
-        X, y, method='lar', Gram=G)
+    alphas_, _, coef_path_ = linear_model.lars_path(X, y, method="lar", Gram=G)
     alpha_, _, coef = linear_model.lars_path(
-        X, y, method='lar', Gram=G, return_path=False)
+        X, y, method="lar", Gram=G, return_path=False
+    )
 
     assert_array_almost_equal(coef, coef_path_[:, -1])
     assert alpha_ == alphas_[-1]
@@ -513,8 +512,9 @@ def test_lars_path_positive_constraint():
 
     err_msg = "Positive constraint not supported for 'lar' coding method."
     with pytest.raises(ValueError, match=err_msg):
-        linear_model.lars_path(diabetes['data'], diabetes['target'],
-                               method='lar', positive=True)
+        linear_model.lars_path(
+            diabetes["data"], diabetes["target"], method="lar", positive=True
+        )
 
     method = 'lasso'
     _, _, coefs = \
@@ -761,3 +761,16 @@ def test_X_none_gram_not_none():
     with pytest.raises(ValueError,
                        match="X cannot be None if Gram is not None"):
         lars_path(X=None, y=[1], Gram='not None')
+
+
+def test_copy_X_with_auto_gram():
+    # Non-regression test for #17789, `copy_X=True` and Gram='auto' does not
+    # overwrite X
+    rng = np.random.RandomState(42)
+    X = rng.rand(6, 6)
+    y = rng.rand(6)
+
+    X_before = X.copy()
+    linear_model.lars_path(X, y, Gram='auto', copy_X=True, method='lasso')
+    # X did not change
+    assert_allclose(X, X_before)
