@@ -1,39 +1,37 @@
 # Author: Wei Xue <xuewei4d@gmail.com>
 #         Thierry Guillemot <thierry.guillemot.work@gmail.com>
-# License: BSD 3 clauseimport warnings
+# License: BSD 3 clause
 
 import sys
+import copy
 import warnings
+import pytest
 
 import numpy as np
-
 from scipy import stats, linalg
 
 from sklearn.covariance import EmpiricalCovariance
-from sklearn.datasets.samples_generator import make_spd_matrix
-from sklearn.externals.six.moves import cStringIO as StringIO
+from sklearn.datasets import make_spd_matrix
+from io import StringIO
 from sklearn.metrics.cluster import adjusted_rand_score
-from sklearn.mixture.gaussian_mixture import GaussianMixture
-from sklearn.mixture.gaussian_mixture import (
+from sklearn.mixture import GaussianMixture
+from sklearn.mixture._gaussian_mixture import (
     _estimate_gaussian_covariances_full,
     _estimate_gaussian_covariances_tied,
     _estimate_gaussian_covariances_diag,
-    _estimate_gaussian_covariances_spherical)
-from sklearn.mixture.gaussian_mixture import _compute_precision_cholesky
-from sklearn.mixture.gaussian_mixture import _compute_log_det_cholesky
+    _estimate_gaussian_covariances_spherical,
+    _compute_precision_cholesky,
+    _compute_log_det_cholesky,
+    )
 from sklearn.exceptions import ConvergenceWarning, NotFittedError
 from sklearn.utils.extmath import fast_logdet
-from sklearn.utils.testing import assert_allclose
-from sklearn.utils.testing import assert_almost_equal
-from sklearn.utils.testing import assert_array_almost_equal
-from sklearn.utils.testing import assert_array_equal
-from sklearn.utils.testing import assert_equal
-from sklearn.utils.testing import assert_greater
-from sklearn.utils.testing import assert_greater_equal
-from sklearn.utils.testing import assert_raise_message
-from sklearn.utils.testing import assert_true
-from sklearn.utils.testing import assert_warns_message
-from sklearn.utils.testing import ignore_warnings
+from sklearn.utils._testing import assert_allclose
+from sklearn.utils._testing import assert_almost_equal
+from sklearn.utils._testing import assert_array_almost_equal
+from sklearn.utils._testing import assert_array_equal
+from sklearn.utils._testing import assert_raise_message
+from sklearn.utils._testing import assert_warns_message
+from sklearn.utils._testing import ignore_warnings
 
 
 COVARIANCE_TYPE = ['full', 'tied', 'diag', 'spherical']
@@ -68,8 +66,8 @@ def generate_data(n_samples, n_features, weights, means, precisions,
     return X
 
 
-class RandomData(object):
-    def __init__(self, rng, n_samples=500, n_components=2, n_features=2,
+class RandomData:
+    def __init__(self, rng, n_samples=200, n_components=2, n_features=2,
                  scale=50):
         self.n_samples = n_samples
         self.n_components = n_components
@@ -95,7 +93,8 @@ class RandomData(object):
         self.X = dict(zip(COVARIANCE_TYPE, [generate_data(
             n_samples, n_features, self.weights, self.means, self.covariances,
             covar_type) for covar_type in COVARIANCE_TYPE]))
-        self.Y = np.hstack([k * np.ones(int(np.round(w * n_samples)))
+        self.Y = np.hstack([np.full(int(np.round(w * n_samples)), k,
+                                    dtype=int)
                             for k, w in enumerate(self.weights)])
 
 
@@ -164,17 +163,17 @@ def test_gaussian_mixture_attributes():
                           covariance_type=covariance_type,
                           init_params=init_params).fit(X)
 
-    assert_equal(gmm.n_components, n_components)
-    assert_equal(gmm.covariance_type, covariance_type)
-    assert_equal(gmm.tol, tol)
-    assert_equal(gmm.reg_covar, reg_covar)
-    assert_equal(gmm.max_iter, max_iter)
-    assert_equal(gmm.n_init, n_init)
-    assert_equal(gmm.init_params, init_params)
+    assert gmm.n_components == n_components
+    assert gmm.covariance_type == covariance_type
+    assert gmm.tol == tol
+    assert gmm.reg_covar == reg_covar
+    assert gmm.max_iter == max_iter
+    assert gmm.n_init == n_init
+    assert gmm.init_params == init_params
 
 
 def test_check_X():
-    from sklearn.mixture.base import _check_X
+    from sklearn.mixture._base import _check_X
     rng = np.random.RandomState(0)
 
     n_samples, n_components, n_features = 10, 2, 2
@@ -286,8 +285,8 @@ def test_check_precisions():
     precisions_not_positive = {
         'full': precisions_not_pos,
         'tied': precisions_not_pos[0],
-        'diag': -1. * np.ones((n_components, n_features)),
-        'spherical': -1. * np.ones(n_components)}
+        'diag': np.full((n_components, n_features), -1.),
+        'spherical': np.full(n_components, -1.)}
 
     not_positive_errors = {
         'full': 'symmetric, positive-definite',
@@ -471,7 +470,7 @@ def _naive_lmvnpdf_diag(X, means, covars):
 
 
 def test_gaussian_mixture_log_probabilities():
-    from sklearn.mixture.gaussian_mixture import _estimate_log_gaussian_prob
+    from sklearn.mixture._gaussian_mixture import _estimate_log_gaussian_prob
 
     # test against with _naive_lmvnpdf_diag
     rng = np.random.RandomState(0)
@@ -560,13 +559,50 @@ def test_gaussian_mixture_predict_predict_proba():
         assert_raise_message(NotFittedError,
                              "This GaussianMixture instance is not fitted "
                              "yet. Call 'fit' with appropriate arguments "
-                             "before using this method.", g.predict, X)
+                             "before using this estimator.", g.predict, X)
 
         g.fit(X)
         Y_pred = g.predict(X)
         Y_pred_proba = g.predict_proba(X).argmax(axis=1)
         assert_array_equal(Y_pred, Y_pred_proba)
-        assert_greater(adjusted_rand_score(Y, Y_pred), .95)
+        assert adjusted_rand_score(Y, Y_pred) > .95
+
+
+@pytest.mark.filterwarnings("ignore:.*did not converge.*")
+@pytest.mark.parametrize('seed, max_iter, tol', [
+    (0, 2, 1e-7),    # strict non-convergence
+    (1, 2, 1e-1),    # loose non-convergence
+    (3, 300, 1e-7),  # strict convergence
+    (4, 300, 1e-1),  # loose convergence
+])
+def test_gaussian_mixture_fit_predict(seed, max_iter, tol):
+    rng = np.random.RandomState(seed)
+    rand_data = RandomData(rng)
+    for covar_type in COVARIANCE_TYPE:
+        X = rand_data.X[covar_type]
+        Y = rand_data.Y
+        g = GaussianMixture(n_components=rand_data.n_components,
+                            random_state=rng, weights_init=rand_data.weights,
+                            means_init=rand_data.means,
+                            precisions_init=rand_data.precisions[covar_type],
+                            covariance_type=covar_type,
+                            max_iter=max_iter, tol=tol)
+
+        # check if fit_predict(X) is equivalent to fit(X).predict(X)
+        f = copy.deepcopy(g)
+        Y_pred1 = f.fit(X).predict(X)
+        Y_pred2 = g.fit_predict(X)
+        assert_array_equal(Y_pred1, Y_pred2)
+        assert adjusted_rand_score(Y, Y_pred2) > .95
+
+
+def test_gaussian_mixture_fit_predict_n_init():
+    # Check that fit_predict is equivalent to fit.predict, when n_init > 1
+    X = np.random.RandomState(0).randn(1000, 5)
+    gm = GaussianMixture(n_components=5, n_init=5, random_state=0)
+    y_pred1 = gm.fit_predict(X)
+    y_pred2 = gm.predict(X)
+    assert_array_equal(y_pred1, y_pred2)
 
 
 def test_gaussian_mixture_fit():
@@ -614,7 +650,7 @@ def test_gaussian_mixture_fit():
             ecov = EmpiricalCovariance()
             ecov.covariance_ = prec_test[h]
             # the accuracy depends on the number of data and randomness, rng
-            assert_allclose(ecov.error_norm(prec_pred[k]), 0, atol=0.1)
+            assert_allclose(ecov.error_norm(prec_pred[k]), 0, atol=0.15)
 
 
 def test_gaussian_mixture_fit_best_params():
@@ -664,11 +700,11 @@ def test_multiple_init():
     for cv_type in COVARIANCE_TYPE:
         train1 = GaussianMixture(n_components=n_components,
                                  covariance_type=cv_type,
-                                 random_state=rng).fit(X).score(X)
+                                 random_state=0).fit(X).score(X)
         train2 = GaussianMixture(n_components=n_components,
                                  covariance_type=cv_type,
-                                 random_state=rng, n_init=5).fit(X).score(X)
-        assert_greater_equal(train2, train1)
+                                 random_state=0, n_init=5).fit(X).score(X)
+        assert train2 >= train1
 
 
 def test_gaussian_mixture_n_parameters():
@@ -681,7 +717,7 @@ def test_gaussian_mixture_n_parameters():
         g = GaussianMixture(
             n_components=n_components, covariance_type=cv_type,
             random_state=rng).fit(X)
-        assert_equal(g._n_parameters(), n_params[cv_type])
+        assert g._n_parameters() == n_params[cv_type]
 
 
 def test_bic_1d_1component():
@@ -717,8 +753,8 @@ def test_gaussian_mixture_aic_bic():
         bic = (2 * n_samples * sgh +
                np.log(n_samples) * g._n_parameters())
         bound = n_features / np.sqrt(n_samples)
-        assert_true((g.aic(X) - aic) / n_samples < bound)
-        assert_true((g.bic(X) - bic) / n_samples < bound)
+        assert (g.aic(X) - aic) / n_samples < bound
+        assert (g.bic(X) - bic) / n_samples < bound
 
 
 def test_gaussian_mixture_verbose():
@@ -742,9 +778,10 @@ def test_gaussian_mixture_verbose():
             sys.stdout = old_stdout
 
 
-def test_warm_start():
-
-    random_state = 0
+@pytest.mark.filterwarnings('ignore:.*did not converge.*')
+@pytest.mark.parametrize("seed", (0, 1, 2))
+def test_warm_start(seed):
+    random_state = seed
     rng = np.random.RandomState(random_state)
     n_samples, n_features, n_components = 500, 2, 2
     X = rng.rand(n_samples, n_features)
@@ -757,16 +794,14 @@ def test_warm_start():
                         reg_covar=0, random_state=random_state,
                         warm_start=True)
 
-    with warnings.catch_warnings():
-        warnings.simplefilter("ignore", ConvergenceWarning)
-        g.fit(X)
-        score1 = h.fit(X).score(X)
-        score2 = h.fit(X).score(X)
+    g.fit(X)
+    score1 = h.fit(X).score(X)
+    score2 = h.fit(X).score(X)
 
     assert_almost_equal(g.weights_, h.weights_)
     assert_almost_equal(g.means_, h.means_)
     assert_almost_equal(g.precisions_, h.precisions_)
-    assert_greater(score2, score1)
+    assert score2 > score1
 
     # Assert that by using warm_start we can converge to a good solution
     g = GaussianMixture(n_components=n_components, n_init=1,
@@ -776,13 +811,37 @@ def test_warm_start():
                         max_iter=5, reg_covar=0, random_state=random_state,
                         warm_start=True, tol=1e-6)
 
-    with warnings.catch_warnings():
-        warnings.simplefilter("ignore", ConvergenceWarning)
-        g.fit(X)
-        h.fit(X).fit(X)
+    g.fit(X)
+    assert not g.converged_
 
-    assert_true(not g.converged_)
-    assert_true(h.converged_)
+    h.fit(X)
+    # depending on the data there is large variability in the number of
+    # refit necessary to converge due to the complete randomness of the
+    # data
+    for _ in range(1000):
+        h.fit(X)
+        if h.converged_:
+            break
+    assert h.converged_
+
+
+@ignore_warnings(category=ConvergenceWarning)
+def test_convergence_detected_with_warm_start():
+    # We check that convergence is detected when warm_start=True
+    rng = np.random.RandomState(0)
+    rand_data = RandomData(rng)
+    n_components = rand_data.n_components
+    X = rand_data.X['full']
+
+    for max_iter in (1, 2, 50):
+        gmm = GaussianMixture(n_components=n_components, warm_start=True,
+                              max_iter=max_iter, random_state=rng)
+        for _ in range(100):
+            gmm.fit(X)
+            if gmm.converged_:
+                break
+        assert gmm.converged_
+        assert max_iter >= gmm.n_iter_
 
 
 def test_score():
@@ -799,7 +858,7 @@ def test_score():
     assert_raise_message(NotFittedError,
                          "This GaussianMixture instance is not fitted "
                          "yet. Call 'fit' with appropriate arguments "
-                         "before using this method.", gmm1.score, X)
+                         "before using this estimator.", gmm1.score, X)
 
     # Check score value
     with warnings.catch_warnings():
@@ -813,7 +872,7 @@ def test_score():
     gmm2 = GaussianMixture(n_components=n_components, n_init=1, reg_covar=0,
                            random_state=rng,
                            covariance_type=covar_type).fit(X)
-    assert_greater(gmm2.score(X), gmm1.score(X))
+    assert gmm2.score(X) > gmm1.score(X)
 
 
 def test_score_samples():
@@ -829,10 +888,10 @@ def test_score_samples():
     assert_raise_message(NotFittedError,
                          "This GaussianMixture instance is not fitted "
                          "yet. Call 'fit' with appropriate arguments "
-                         "before using this method.", gmm.score_samples, X)
+                         "before using this estimator.", gmm.score_samples, X)
 
     gmm_score_samples = gmm.fit(X).score_samples(X)
-    assert_equal(gmm_score_samples.shape[0], rand_data.n_samples)
+    assert gmm_score_samples.shape[0] == rand_data.n_samples
 
 
 def test_monotonic_likelihood():
@@ -855,17 +914,13 @@ def test_monotonic_likelihood():
             # training log likelihood increases after each iteration.
             for _ in range(600):
                 prev_log_likelihood = current_log_likelihood
-                try:
-                    current_log_likelihood = gmm.fit(X).score(X)
-                except ConvergenceWarning:
-                    pass
-                assert_greater_equal(current_log_likelihood,
-                                     prev_log_likelihood)
+                current_log_likelihood = gmm.fit(X).score(X)
+                assert current_log_likelihood >= prev_log_likelihood
 
                 if gmm.converged_:
                     break
 
-            assert_true(gmm.converged_)
+            assert gmm.converged_
 
 
 def test_regularisation():
@@ -960,24 +1015,25 @@ def test_sample():
 
         # Check shapes of sampled data, see
         # https://github.com/scikit-learn/scikit-learn/issues/7701
-        assert_equal(X_s.shape, (n_samples, n_features))
+        assert X_s.shape == (n_samples, n_features)
 
         for sample_size in range(1, 100):
             X_s, _ = gmm.sample(sample_size)
-            assert_equal(X_s.shape, (sample_size, n_features))
+            assert X_s.shape == (sample_size, n_features)
 
 
 @ignore_warnings(category=ConvergenceWarning)
 def test_init():
     # We check that by increasing the n_init number we have a better solution
-    random_state = 0
-    rand_data = RandomData(np.random.RandomState(random_state), scale=1)
-    n_components = rand_data.n_components
-    X = rand_data.X['full']
+    for random_state in range(15):
+        rand_data = RandomData(np.random.RandomState(random_state),
+                               n_samples=50, scale=1)
+        n_components = rand_data.n_components
+        X = rand_data.X['full']
 
-    gmm1 = GaussianMixture(n_components=n_components, n_init=1,
-                           max_iter=1, random_state=random_state).fit(X)
-    gmm2 = GaussianMixture(n_components=n_components, n_init=100,
-                           max_iter=1, random_state=random_state).fit(X)
+        gmm1 = GaussianMixture(n_components=n_components, n_init=1,
+                               max_iter=1, random_state=random_state).fit(X)
+        gmm2 = GaussianMixture(n_components=n_components, n_init=10,
+                               max_iter=1, random_state=random_state).fit(X)
 
-    assert_greater(gmm2.lower_bound_, gmm1.lower_bound_)
+        assert gmm2.lower_bound_ >= gmm1.lower_bound_
